# Валидация графа
Одной из особенностей библиотеки является возможность построения графа зависимостей после регистрации компонентов. 

> На текущий момент граф нельзя получить, но в ближайших версия API будет дополнено, и станет возможным получить граф зависимостей для собственного использования. А также будет изменен способ проверки графа зависимостей.

На основе этой возможности библиотека предлагает проверить этот граф, до начала использования. То есть можно зарегистрировать все зависимости, и дальше вызовом одной функции убедится, что все зарегистрировано правильно.

Эта функция называется `validate` и находится в контейнере:
```Swift
container.register(...)
container.register(...)
...

#if DEBUG
if !container.validate(checkGraphCycles: true) {
    fatalError("invalid graph")
}
#endif
```
Подобная валидация не имеет смысла в релизной версии, так как граф зависимостей не меняется от запуска к запуску, что значит - если в дебаге проверка прошла, значит и в релизе тоже пройдет.

Функция валидации проверяет граф на то, что все вершины достижимы, что каждый объект можно создать. Булева переменная `checkGraphCycles` добавляет к проверки также проверку циклов. Но проверка циклов занимает очень много времени (отчасти из-за не оптимальности алгоритмов) и не стоит её вызывать без лишней необходимости.

Все возникшие проблемы пишутся в лог. О том, какие могут возникнуть ошибки можно прочитать на странице посвященной [логированию](logs.md#Описание-логов) - все ошибки которые происходят при валидации графа помечены специальным образом `(validate)`.
